# [[[ HEADER ]]]
package Perl::Object;
use strict;
use warnings;
our $VERSION = 0.003_000;

# NEED FIX?: Perl Object, no inheritance from Perl or Perl::Class

# [[[ OO INHERITANCE ]]]
#use parent qw(Perl::Class);
#use Perl::Class;

1;  # end of class
