using std::cout;  using std::cerr;  using std::endl;  using std::to_string;

#ifndef __CPP__INCLUDED__Perl__Structure__Array__SubTypes3D_cpp
#define __CPP__INCLUDED__Perl__Structure__Array__SubTypes3D_cpp 0.001_000

#include <Perl/Structure/Array/SubTypes3D.h>  // -> ??? (relies on <vector> being included via Inline::CPP's AUTO_INCLUDE config option in RPerl/Inline.pm)

// NEED ADD CODE HERE
// NEED ADD CODE HERE
// NEED ADD CODE HERE

#endif
