# [[[ HEADER ]]]
package    # hide from PAUSE indexing
    Perl;
use strict;
use warnings;
our $VERSION = 0.001_000;

# [[[ OO INHERITANCE ]]]
use parent qw(Perl::Class);
use Perl::Class;

1;  # end of package
