using std::cout;  using std::cerr;  using std::endl;  using std::to_string;

#ifndef __CPP__INCLUDED__Perl__Structure__Array_cpp
#define __CPP__INCLUDED__Perl__Structure__Array_cpp 0.008_000

#include <Perl/Structure/Array.h>		// -> NULL (relies on <vector> being included via Inline::CPP's AUTO_INCLUDE config option)

#endif
