# THIS FILE IS AUTOMATICALLY GENERATED BY   script/development/perlapinames_regenerate.pl
# DO NOT EDIT THIS FILE DIRECTLY!!!  please put all changes in perlapinames_regenerate.pl
## no critic qw(Capitalization ProhibitMultiplePackages ProhibitReusedNames)  # SYSTEM DEFAULT 3: allow multiple & lower case package names
package    # hide from PAUSE indexing
    perlapinames_generated;
use strict;
use warnings;
our $VERSION = 0.001_000;

## no critic qw(ProhibitParensWithBuiltins ProhibitNoisyQuotes)  # SYSTEM SPECIAL 3: allow auto-generated code

$perlapinames_generated::FUNCTIONS_DOCUMENTED = undef;
$perlapinames_generated::FUNCTIONS_DOCUMENTED = {
    'AvFILL'                                  => 1,
    'BOM_UTF8'                                => 1,
    'BhkDISABLE'                              => 1,
    'BhkENABLE'                               => 1,
    'BhkENTRY_set'                            => 1,
    'CLASS'                                   => 1,
    'Copy'                                    => 1,
    'CopyD'                                   => 1,
    'CvPADLIST'                               => 1,
    'CvSTASH'                                 => 1,
    'DECLARATION_FOR_LC_NUMERIC_MANIPULATION' => 1,
    'DO_UTF8'                                 => 1,
    'ENTER'                                   => 1,
    'EXTEND'                                  => 1,
    'FREETMPS'                                => 1,
    'GIMME'                                   => 1,
    'GIMME_V'                                 => 1,
    'G_ARRAY'                                 => 1,
    'G_DISCARD'                               => 1,
    'G_EVAL'                                  => 1,
    'G_NOARGS'                                => 1,
    'G_SCALAR'                                => 1,
    'G_VOID'                                  => 1,
    'GvAV'                                    => 1,
    'GvCV'                                    => 1,
    'GvHV'                                    => 1,
    'GvSV'                                    => 1,
    'HEf_SVKEY'                               => 1,
    'HeHASH'                                  => 1,
    'HeKEY'                                   => 1,
    'HeKLEN'                                  => 1,
    'HePV'                                    => 1,
    'HeSVKEY'                                 => 1,
    'HeSVKEY_force'                           => 1,
    'HeSVKEY_set'                             => 1,
    'HeUTF8'                                  => 1,
    'HeVAL'                                   => 1,
    'HvENAME'                                 => 1,
    'HvENAMELEN'                              => 1,
    'HvENAMEUTF8'                             => 1,
    'HvNAME'                                  => 1,
    'HvNAMELEN'                               => 1,
    'HvNAMEUTF8'                              => 1,
    'LEAVE'                                   => 1,
    'LINKLIST'                                => 1,
    'MARK'                                    => 1,
    'MULTICALL'                               => 1,
    'Move'                                    => 1,
    'MoveD'                                   => 1,
    'Newx'                                    => 1,
    'Newxc'                                   => 1,
    'Newxz'                                   => 1,
    'Nullav'                                  => 1,
    'Nullch'                                  => 1,
    'Nullcv'                                  => 1,
    'Nullhv'                                  => 1,
    'Nullsv'                                  => 1,
    'OP_CLASS'                                => 1,
    'OP_DESC'                                 => 1,
    'OP_NAME'                                 => 1,
    'OP_TYPE_IS'                              => 1,
    'OP_TYPE_IS_OR_WAS'                       => 1,
    'ORIGMARK'                                => 1,
    'OpHAS_SIBLING'                           => 1,
    'OpLASTSIB_set'                           => 1,
    'OpMAYBESIB_set'                          => 1,
    'OpMORESIB_set'                           => 1,
    'OpSIBLING'                               => 1,
    'PERL_SYS_INIT'                           => 1,
    'PERL_SYS_INIT3'                          => 1,
    'PERL_SYS_TERM'                           => 1,
    'PL_check'                                => 1,
    'PL_comppad'                              => 1,
    'PL_comppad_name'                         => 1,
    'PL_curpad'                               => 1,
    'PL_keyword_plugin'                       => 1,
    'PL_modglobal'                            => 1,
    'PL_na'                                   => 1,
    'PL_opfreehook'                           => 1,
    'PL_parser'                               => 1,
    'PL_peepp'                                => 1,
    'PL_rpeepp'                               => 1,
    'PL_sv_no'                                => 1,
    'PL_sv_undef'                             => 1,
    'PL_sv_yes'                               => 1,
    'POP_MULTICALL'                           => 1,
    'POPi'                                    => 1,
    'POPl'                                    => 1,
    'POPn'                                    => 1,
    'POPp'                                    => 1,
    'POPpbytex'                               => 1,
    'POPpx'                                   => 1,
    'POPs'                                    => 1,
    'POPu'                                    => 1,
    'POPul'                                   => 1,
    'PUSHMARK'                                => 1,
    'PUSH_MULTICALL'                          => 1,
    'PUSHi'                                   => 1,
    'PUSHmortal'                              => 1,
    'PUSHn'                                   => 1,
    'PUSHp'                                   => 1,
    'PUSHs'                                   => 1,
    'PUSHu'                                   => 1,
    'PUTBACK'                                 => 1,
    'PadARRAY'                                => 1,
    'PadMAX'                                  => 1,
    'PadlistARRAY'                            => 1,
    'PadlistMAX'                              => 1,
    'PadlistNAMES'                            => 1,
    'PadlistNAMESARRAY'                       => 1,
    'PadlistNAMESMAX'                         => 1,
    'PadlistREFCNT'                           => 1,
    'PadnameLEN'                              => 1,
    'PadnamePV'                               => 1,
    'PadnameREFCNT'                           => 1,
    'PadnameREFCNT_dec'                       => 1,
    'PadnameSV'                               => 1,
    'PadnameUTF8'                             => 1,
    'PadnamelistARRAY'                        => 1,
    'PadnamelistMAX'                          => 1,
    'PadnamelistREFCNT'                       => 1,
    'PadnamelistREFCNT_dec'                   => 1,
    'Perl_signbit'                            => 1,
    'Poison'                                  => 1,
    'PoisonFree'                              => 1,
    'PoisonNew'                               => 1,
    'PoisonWith'                              => 1,
    'READ_XDIGIT'                             => 1,
    'REPLACEMENT_CHARACTER_UTF8'              => 1,
    'RESTORE_LC_NUMERIC'                      => 1,
    'RETVAL'                                  => 1,
    'Renew'                                   => 1,
    'Renewc'                                  => 1,
    'SAVETMPS'                                => 1,
    'SP'                                      => 1,
    'SPAGAIN'                                 => 1,
    'ST'                                      => 1,
    'STORE_LC_NUMERIC_FORCE_TO_UNDERLYING'    => 1,
    'STORE_LC_NUMERIC_SET_TO_NEEDED'          => 1,
    'SVt_INVLIST'                             => 1,
    'SVt_IV'                                  => 1,
    'SVt_NULL'                                => 1,
    'SVt_NV'                                  => 1,
    'SVt_PV'                                  => 1,
    'SVt_PVAV'                                => 1,
    'SVt_PVCV'                                => 1,
    'SVt_PVFM'                                => 1,
    'SVt_PVGV'                                => 1,
    'SVt_PVHV'                                => 1,
    'SVt_PVIO'                                => 1,
    'SVt_PVIV'                                => 1,
    'SVt_PVLV'                                => 1,
    'SVt_PVMG'                                => 1,
    'SVt_PVNV'                                => 1,
    'SVt_REGEXP'                              => 1,
    'Safefree'                                => 1,
    'StructCopy'                              => 1,
    'SvCUR'                                   => 1,
    'SvCUR_set'                               => 1,
    'SvEND'                                   => 1,
    'SvGAMAGIC'                               => 1,
    'SvGETMAGIC'                              => 1,
    'SvGROW'                                  => 1,
    'SvIOK'                                   => 1,
    'SvIOK_UV'                                => 1,
    'SvIOK_notUV'                             => 1,
    'SvIOK_off'                               => 1,
    'SvIOK_on'                                => 1,
    'SvIOK_only'                              => 1,
    'SvIOK_only_UV'                           => 1,
    'SvIOKp'                                  => 1,
    'SvIV'                                    => 1,
    'SvIVX'                                   => 1,
    'SvIV_nomg'                               => 1,
    'SvIV_set'                                => 1,
    'SvIVx'                                   => 1,
    'SvIsCOW'                                 => 1,
    'SvIsCOW_shared_hash'                     => 1,
    'SvLEN'                                   => 1,
    'SvLEN_set'                               => 1,
    'SvLOCK'                                  => 1,
    'SvMAGIC_set'                             => 1,
    'SvNIOK'                                  => 1,
    'SvNIOK_off'                              => 1,
    'SvNIOKp'                                 => 1,
    'SvNOK'                                   => 1,
    'SvNOK_off'                               => 1,
    'SvNOK_on'                                => 1,
    'SvNOK_only'                              => 1,
    'SvNOKp'                                  => 1,
    'SvNV'                                    => 1,
    'SvNVX'                                   => 1,
    'SvNV_nomg'                               => 1,
    'SvNV_set'                                => 1,
    'SvNVx'                                   => 1,
    'SvOK'                                    => 1,
    'SvOOK'                                   => 1,
    'SvOOK_offset'                            => 1,
    'SvPOK'                                   => 1,
    'SvPOK_off'                               => 1,
    'SvPOK_on'                                => 1,
    'SvPOK_only'                              => 1,
    'SvPOK_only_UTF8'                         => 1,
    'SvPOKp'                                  => 1,
    'SvPV'                                    => 1,
    'SvPVCLEAR'                               => 1,
    'SvPVX'                                   => 1,
    'SvPV_force'                              => 1,
    'SvPV_force_nomg'                         => 1,
    'SvPV_nolen'                              => 1,
    'SvPV_nomg'                               => 1,
    'SvPV_nomg_nolen'                         => 1,
    'SvPV_set'                                => 1,
    'SvPVbyte'                                => 1,
    'SvPVbyte_force'                          => 1,
    'SvPVbyte_nolen'                          => 1,
    'SvPVbytex'                               => 1,
    'SvPVbytex_force'                         => 1,
    'SvPVutf8'                                => 1,
    'SvPVutf8_force'                          => 1,
    'SvPVutf8_nolen'                          => 1,
    'SvPVutf8x'                               => 1,
    'SvPVutf8x_force'                         => 1,
    'SvPVx'                                   => 1,
    'SvREADONLY'                              => 1,
    'SvREADONLY_off'                          => 1,
    'SvREADONLY_on'                           => 1,
    'SvREFCNT'                                => 1,
    'SvREFCNT_dec'                            => 1,
    'SvREFCNT_dec_NN'                         => 1,
    'SvREFCNT_inc'                            => 1,
    'SvREFCNT_inc_NN'                         => 1,
    'SvREFCNT_inc_simple'                     => 1,
    'SvREFCNT_inc_simple_NN'                  => 1,
    'SvREFCNT_inc_simple_void'                => 1,
    'SvREFCNT_inc_simple_void_NN'             => 1,
    'SvREFCNT_inc_void'                       => 1,
    'SvREFCNT_inc_void_NN'                    => 1,
    'SvROK'                                   => 1,
    'SvROK_off'                               => 1,
    'SvROK_on'                                => 1,
    'SvRV'                                    => 1,
    'SvRV_set'                                => 1,
    'SvRX'                                    => 1,
    'SvRXOK'                                  => 1,
    'SvSETMAGIC'                              => 1,
    'SvSHARE'                                 => 1,
    'SvSTASH'                                 => 1,
    'SvSTASH_set'                             => 1,
    'SvSetMagicSV'                            => 1,
    'SvSetMagicSV_nosteal'                    => 1,
    'SvSetSV'                                 => 1,
    'SvSetSV_nosteal'                         => 1,
    'SvTAINT'                                 => 1,
    'SvTAINTED'                               => 1,
    'SvTAINTED_off'                           => 1,
    'SvTAINTED_on'                            => 1,
    'SvTRUE'                                  => 1,
    'SvTRUE_nomg'                             => 1,
    'SvTYPE'                                  => 1,
    'SvUNLOCK'                                => 1,
    'SvUOK'                                   => 1,
    'SvUPGRADE'                               => 1,
    'SvUTF8'                                  => 1,
    'SvUTF8_off'                              => 1,
    'SvUTF8_on'                               => 1,
    'SvUV'                                    => 1,
    'SvUVX'                                   => 1,
    'SvUV_nomg'                               => 1,
    'SvUV_set'                                => 1,
    'SvUVx'                                   => 1,
    'SvVOK'                                   => 1,
    'THIS'                                    => 1,
    'UNDERBAR'                                => 1,
    'UTF8SKIP'                                => 1,
    'UTF8_IS_INVARIANT'                       => 1,
    'UTF8_IS_NONCHAR'                         => 1,
    'UTF8_IS_SUPER'                           => 1,
    'UTF8_IS_SURROGATE'                       => 1,
    'UVCHR_IS_INVARIANT'                      => 1,
    'UVCHR_SKIP'                              => 1,
    'XCPT_CATCH'                              => 1,
    'XCPT_RETHROW'                            => 1,
    'XCPT_TRY_END'                            => 1,
    'XCPT_TRY_START'                          => 1,
    'XPUSHi'                                  => 1,
    'XPUSHmortal'                             => 1,
    'XPUSHn'                                  => 1,
    'XPUSHp'                                  => 1,
    'XPUSHs'                                  => 1,
    'XPUSHu'                                  => 1,
    'XS'                                      => 1,
    'XSRETURN'                                => 1,
    'XSRETURN_EMPTY'                          => 1,
    'XSRETURN_IV'                             => 1,
    'XSRETURN_NO'                             => 1,
    'XSRETURN_NV'                             => 1,
    'XSRETURN_PV'                             => 1,
    'XSRETURN_UNDEF'                          => 1,
    'XSRETURN_UV'                             => 1,
    'XSRETURN_YES'                            => 1,
    'XST_mIV'                                 => 1,
    'XST_mNO'                                 => 1,
    'XST_mNV'                                 => 1,
    'XST_mPV'                                 => 1,
    'XST_mUNDEF'                              => 1,
    'XST_mYES'                                => 1,
    'XS_APIVERSION_BOOTCHECK'                 => 1,
    'XS_EXTERNAL'                             => 1,
    'XS_INTERNAL'                             => 1,
    'XS_VERSION'                              => 1,
    'XS_VERSION_BOOTCHECK'                    => 1,
    'XopDISABLE'                              => 1,
    'XopENABLE'                               => 1,
    'XopENTRY'                                => 1,
    'XopENTRYCUSTOM'                          => 1,
    'XopENTRY_set'                            => 1,
    'XopFLAGS'                                => 1,
    'Zero'                                    => 1,
    'ZeroD'                                   => 1,
    'alloccopstash'                           => 1,
    'av_clear'                                => 1,
    'av_create_and_push'                      => 1,
    'av_create_and_unshift_one'               => 1,
    'av_delete'                               => 1,
    'av_exists'                               => 1,
    'av_extend'                               => 1,
    'av_fetch'                                => 1,
    'av_fill'                                 => 1,
    'av_len'                                  => 1,
    'av_make'                                 => 1,
    'av_pop'                                  => 1,
    'av_push'                                 => 1,
    'av_shift'                                => 1,
    'av_store'                                => 1,
    'av_tindex'                               => 1,
    'av_top_index'                            => 1,
    'av_undef'                                => 1,
    'av_unshift'                              => 1,
    'ax'                                      => 1,
    'block_end'                               => 1,
    'block_start'                             => 1,
    'blockhook_register'                      => 1,
    'boolSV'                                  => 1,
    'bytes_cmp_utf8'                          => 1,
    'bytes_from_utf8'                         => 1,
    'bytes_to_utf8'                           => 1,
    'call_argv'                               => 1,
    'call_method'                             => 1,
    'call_pv'                                 => 1,
    'call_sv'                                 => 1,
    'caller_cx'                               => 1,
    'ckWARN'                                  => 1,
    'ckWARN2'                                 => 1,
    'ckWARN2_d'                               => 1,
    'ckWARN3'                                 => 1,
    'ckWARN3_d'                               => 1,
    'ckWARN4'                                 => 1,
    'ckWARN4_d'                               => 1,
    'ckWARN_d'                                => 1,
    'ck_entersub_args_list'                   => 1,
    'ck_entersub_args_proto'                  => 1,
    'ck_entersub_args_proto_or_list'          => 1,
    'cop_fetch_label'                         => 1,
    'cop_hints_2hv'                           => 1,
    'cop_hints_fetch_pv'                      => 1,
    'cop_hints_fetch_pvn'                     => 1,
    'cop_hints_fetch_pvs'                     => 1,
    'cop_hints_fetch_sv'                      => 1,
    'cop_store_label'                         => 1,
    'cophh_2hv'                               => 1,
    'cophh_copy'                              => 1,
    'cophh_delete_pv'                         => 1,
    'cophh_delete_pvn'                        => 1,
    'cophh_delete_pvs'                        => 1,
    'cophh_delete_sv'                         => 1,
    'cophh_fetch_pv'                          => 1,
    'cophh_fetch_pvn'                         => 1,
    'cophh_fetch_pvs'                         => 1,
    'cophh_fetch_sv'                          => 1,
    'cophh_free'                              => 1,
    'cophh_new_empty'                         => 1,
    'cophh_store_pv'                          => 1,
    'cophh_store_pvn'                         => 1,
    'cophh_store_pvs'                         => 1,
    'cophh_store_sv'                          => 1,
    'croak'                                   => 1,
    'croak_no_modify'                         => 1,
    'croak_sv'                                => 1,
    'croak_xs_usage'                          => 1,
    'custom_op_desc'                          => 1,
    'custom_op_name'                          => 1,
    'custom_op_register'                      => 1,
    'custom_op_xop'                           => 1,
    'cv_clone'                                => 1,
    'cv_const_sv'                             => 1,
    'cv_get_call_checker'                     => 1,
    'cv_name'                                 => 1,
    'cv_set_call_checker'                     => 1,
    'cv_set_call_checker_flags'               => 1,
    'cv_undef'                                => 1,
    'dAX'                                     => 1,
    'dAXMARK'                                 => 1,
    'dITEMS'                                  => 1,
    'dMARK'                                   => 1,
    'dMULTICALL'                              => 1,
    'dORIGMARK'                               => 1,
    'dSP'                                     => 1,
    'dUNDERBAR'                               => 1,
    'dXCPT'                                   => 1,
    'dXSARGS'                                 => 1,
    'dXSI32'                                  => 1,
    'die'                                     => 1,
    'die_sv'                                  => 1,
    'dump_all'                                => 1,
    'dump_c_backtrace'                        => 1,
    'dump_packsubs'                           => 1,
    'eval_pv'                                 => 1,
    'eval_sv'                                 => 1,
    'fbm_compile'                             => 1,
    'fbm_instr'                               => 1,
    'find_runcv'                              => 1,
    'find_rundefsv'                           => 1,
    'find_rundefsvoffset'                     => 1,
    'foldEQ'                                  => 1,
    'foldEQ_locale'                           => 1,
    'foldEQ_utf8'                             => 1,
    'form'                                    => 1,
    'get_av'                                  => 1,
    'get_c_backtrace_dump'                    => 1,
    'get_cv'                                  => 1,
    'get_cvn_flags'                           => 1,
    'get_hv'                                  => 1,
    'get_sv'                                  => 1,
    'getcwd_sv'                               => 1,
    'grok_bin'                                => 1,
    'grok_hex'                                => 1,
    'grok_infnan'                             => 1,
    'grok_number'                             => 1,
    'grok_number_flags'                       => 1,
    'grok_numeric_radix'                      => 1,
    'grok_oct'                                => 1,
    'gv_const_sv'                             => 1,
    'gv_fetchmeth'                            => 1,
    'gv_fetchmeth_autoload'                   => 1,
    'gv_fetchmeth_pv'                         => 1,
    'gv_fetchmeth_pv_autoload'                => 1,
    'gv_fetchmeth_pvn'                        => 1,
    'gv_fetchmeth_pvn_autoload'               => 1,
    'gv_fetchmeth_sv'                         => 1,
    'gv_fetchmeth_sv_autoload'                => 1,
    'gv_fetchmethod'                          => 1,
    'gv_fetchmethod_autoload'                 => 1,
    'gv_init'                                 => 1,
    'gv_init_pv'                              => 1,
    'gv_init_pvn'                             => 1,
    'gv_init_sv'                              => 1,
    'gv_stashpv'                              => 1,
    'gv_stashpvn'                             => 1,
    'gv_stashpvs'                             => 1,
    'gv_stashsv'                              => 1,
    'hv_assert'                               => 1,
    'hv_bucket_ratio'                         => 1,
    'hv_clear'                                => 1,
    'hv_clear_placeholders'                   => 1,
    'hv_copy_hints_hv'                        => 1,
    'hv_delete'                               => 1,
    'hv_delete_ent'                           => 1,
    'hv_exists'                               => 1,
    'hv_exists_ent'                           => 1,
    'hv_fetch'                                => 1,
    'hv_fetch_ent'                            => 1,
    'hv_fetchs'                               => 1,
    'hv_fill'                                 => 1,
    'hv_iterinit'                             => 1,
    'hv_iterkey'                              => 1,
    'hv_iterkeysv'                            => 1,
    'hv_iternext'                             => 1,
    'hv_iternext_flags'                       => 1,
    'hv_iternextsv'                           => 1,
    'hv_iterval'                              => 1,
    'hv_magic'                                => 1,
    'hv_scalar'                               => 1,
    'hv_store'                                => 1,
    'hv_store_ent'                            => 1,
    'hv_stores'                               => 1,
    'hv_undef'                                => 1,
    'ibcmp'                                   => 1,
    'ibcmp_locale'                            => 1,
    'intro_my'                                => 1,
    'isALPHA'                                 => 1,
    'isALPHANUMERIC'                          => 1,
    'isASCII'                                 => 1,
    'isBLANK'                                 => 1,
    'isC9_STRICT_UTF8_CHAR'                   => 1,
    'isCNTRL'                                 => 1,
    'isDIGIT'                                 => 1,
    'isGRAPH'                                 => 1,
    'isIDCONT'                                => 1,
    'isIDFIRST'                               => 1,
    'isLOWER'                                 => 1,
    'isOCTAL'                                 => 1,
    'isPRINT'                                 => 1,
    'isPSXSPC'                                => 1,
    'isPUNCT'                                 => 1,
    'isSPACE'                                 => 1,
    'isSTRICT_UTF8_CHAR'                      => 1,
    'isUPPER'                                 => 1,
    'isUTF8_CHAR'                             => 1,
    'isUTF8_CHAR_flags'                       => 1,
    'isWORDCHAR'                              => 1,
    'isXDIGIT'                                => 1,
    'is_ascii_string'                         => 1,
    'is_c9strict_utf8_string'                 => 1,
    'is_c9strict_utf8_string_loc'             => 1,
    'is_c9strict_utf8_string_loclen'          => 1,
    'is_invariant_string'                     => 1,
    'is_safe_syscall'                         => 1,
    'is_strict_utf8_string'                   => 1,
    'is_strict_utf8_string_loc'               => 1,
    'is_strict_utf8_string_loclen'            => 1,
    'is_utf8_char'                            => 1,
    'is_utf8_char_buf'                        => 1,
    'is_utf8_fixed_width_buf_flags'           => 1,
    'is_utf8_fixed_width_buf_loc_flags'       => 1,
    'is_utf8_fixed_width_buf_loclen_flags'    => 1,
    'is_utf8_invariant_string'                => 1,
    'is_utf8_string'                          => 1,
    'is_utf8_string_flags'                    => 1,
    'is_utf8_string_loc'                      => 1,
    'is_utf8_string_loc_flags'                => 1,
    'is_utf8_string_loclen'                   => 1,
    'is_utf8_string_loclen_flags'             => 1,
    'is_utf8_valid_partial_char'              => 1,
    'is_utf8_valid_partial_char_flags'        => 1,
    'isinfnan'                                => 1,
    'items'                                   => 1,
    'ix'                                      => 1,
    'lex_bufutf8'                             => 1,
    'lex_discard_to'                          => 1,
    'lex_grow_linestr'                        => 1,
    'lex_next_chunk'                          => 1,
    'lex_peek_unichar'                        => 1,
    'lex_read_space'                          => 1,
    'lex_read_to'                             => 1,
    'lex_read_unichar'                        => 1,
    'lex_start'                               => 1,
    'lex_stuff_pv'                            => 1,
    'lex_stuff_pvn'                           => 1,
    'lex_stuff_pvs'                           => 1,
    'lex_stuff_sv'                            => 1,
    'lex_unstuff'                             => 1,
    'load_module'                             => 1,
    'looks_like_number'                       => 1,
    'mPUSHi'                                  => 1,
    'mPUSHn'                                  => 1,
    'mPUSHp'                                  => 1,
    'mPUSHs'                                  => 1,
    'mPUSHu'                                  => 1,
    'mXPUSHi'                                 => 1,
    'mXPUSHn'                                 => 1,
    'mXPUSHp'                                 => 1,
    'mXPUSHs'                                 => 1,
    'mXPUSHu'                                 => 1,
    'memEQ'                                   => 1,
    'memNE'                                   => 1,
    'mess'                                    => 1,
    'mess_sv'                                 => 1,
    'mg_clear'                                => 1,
    'mg_copy'                                 => 1,
    'mg_find'                                 => 1,
    'mg_findext'                              => 1,
    'mg_free'                                 => 1,
    'mg_free_type'                            => 1,
    'mg_get'                                  => 1,
    'mg_length'                               => 1,
    'mg_magical'                              => 1,
    'mg_set'                                  => 1,
    'mro_get_linear_isa'                      => 1,
    'mro_method_changed_in'                   => 1,
    'mro_register'                            => 1,
    'my_snprintf'                             => 1,
    'my_sprintf'                              => 1,
    'my_strlcat'                              => 1,
    'my_strlcpy'                              => 1,
    'my_vsnprintf'                            => 1,
    'newASSIGNOP'                             => 1,
    'newAV'                                   => 1,
    'newBINOP'                                => 1,
    'newCONDOP'                               => 1,
    'newCONSTSUB'                             => 1,
    'newCONSTSUB_flags'                       => 1,
    'newDEFSVOP'                              => 1,
    'newFOROP'                                => 1,
    'newGIVENOP'                              => 1,
    'newGVOP'                                 => 1,
    'newHV'                                   => 1,
    'newLISTOP'                               => 1,
    'newLOGOP'                                => 1,
    'newLOOPEX'                               => 1,
    'newLOOPOP'                               => 1,
    'newMETHOP'                               => 1,
    'newMETHOP_named'                         => 1,
    'newNULLLIST'                             => 1,
    'newOP'                                   => 1,
    'newPADNAMELIST'                          => 1,
    'newPADNAMEouter'                         => 1,
    'newPADNAMEpvn'                           => 1,
    'newPADOP'                                => 1,
    'newPMOP'                                 => 1,
    'newPVOP'                                 => 1,
    'newRANGE'                                => 1,
    'newRV_inc'                               => 1,
    'newRV_noinc'                             => 1,
    'newSLICEOP'                              => 1,
    'newSTATEOP'                              => 1,
    'newSV'                                   => 1,
    'newSVOP'                                 => 1,
    'newSV_type'                              => 1,
    'newSVhek'                                => 1,
    'newSViv'                                 => 1,
    'newSVnv'                                 => 1,
    'newSVpadname'                            => 1,
    'newSVpv'                                 => 1,
    'newSVpv_share'                           => 1,
    'newSVpvf'                                => 1,
    'newSVpvn'                                => 1,
    'newSVpvn_flags'                          => 1,
    'newSVpvn_share'                          => 1,
    'newSVpvn_utf8'                           => 1,
    'newSVpvs'                                => 1,
    'newSVpvs_flags'                          => 1,
    'newSVpvs_share'                          => 1,
    'newSVrv'                                 => 1,
    'newSVsv'                                 => 1,
    'newSVuv'                                 => 1,
    'newUNOP'                                 => 1,
    'newUNOP_AUX'                             => 1,
    'newWHENOP'                               => 1,
    'newWHILEOP'                              => 1,
    'newXS'                                   => 1,
    'newXSproto'                              => 1,
    'new_version'                             => 1,
    'ninstr'                                  => 1,
    'nothreadhook'                            => 1,
    'op_append_elem'                          => 1,
    'op_append_list'                          => 1,
    'op_class'                                => 1,
    'op_contextualize'                        => 1,
    'op_convert_list'                         => 1,
    'op_dump'                                 => 1,
    'op_free'                                 => 1,
    'op_linklist'                             => 1,
    'op_lvalue'                               => 1,
    'op_null'                                 => 1,
    'op_parent'                               => 1,
    'op_prepend_elem'                         => 1,
    'op_scope'                                => 1,
    'op_sibling_splice'                       => 1,
    'pack_cat'                                => 1,
    'packlist'                                => 1,
    'pad_add_anon'                            => 1,
    'pad_add_name_pv'                         => 1,
    'pad_add_name_pvn'                        => 1,
    'pad_add_name_pvs'                        => 1,
    'pad_add_name_sv'                         => 1,
    'pad_alloc'                               => 1,
    'pad_compname_type'                       => 1,
    'pad_findmy_pv'                           => 1,
    'pad_findmy_pvn'                          => 1,
    'pad_findmy_pvs'                          => 1,
    'pad_findmy_sv'                           => 1,
    'pad_new'                                 => 1,
    'pad_setsv'                               => 1,
    'pad_sv'                                  => 1,
    'pad_tidy'                                => 1,
    'padnamelist_fetch'                       => 1,
    'padnamelist_store'                       => 1,
    'parse_arithexpr'                         => 1,
    'parse_barestmt'                          => 1,
    'parse_block'                             => 1,
    'parse_fullexpr'                          => 1,
    'parse_fullstmt'                          => 1,
    'parse_label'                             => 1,
    'parse_listexpr'                          => 1,
    'parse_stmtseq'                           => 1,
    'parse_termexpr'                          => 1,
    'perl_alloc'                              => 1,
    'perl_clone'                              => 1,
    'perl_construct'                          => 1,
    'perl_destruct'                           => 1,
    'perl_free'                               => 1,
    'perl_parse'                              => 1,
    'perl_run'                                => 1,
    'prescan_version'                         => 1,
    'pv_display'                              => 1,
    'pv_escape'                               => 1,
    'pv_pretty'                               => 1,
    'pv_uni_display'                          => 1,
    'quadmath_format_needed'                  => 1,
    'quadmath_format_single'                  => 1,
    'require_pv'                              => 1,
    'rninstr'                                 => 1,
    'rv2cv_op_cv'                             => 1,
    'save_gp'                                 => 1,
    'savepv'                                  => 1,
    'savepvn'                                 => 1,
    'savepvs'                                 => 1,
    'savesharedpv'                            => 1,
    'savesharedpvn'                           => 1,
    'savesharedpvs'                           => 1,
    'savesharedsvpv'                          => 1,
    'savesvpv'                                => 1,
    'scan_bin'                                => 1,
    'scan_hex'                                => 1,
    'scan_oct'                                => 1,
    'scan_version'                            => 1,
    'setdefout'                               => 1,
    'sortsv'                                  => 1,
    'sortsv_flags'                            => 1,
    'strEQ'                                   => 1,
    'strGE'                                   => 1,
    'strGT'                                   => 1,
    'strLE'                                   => 1,
    'strLT'                                   => 1,
    'strNE'                                   => 1,
    'strnEQ'                                  => 1,
    'strnNE'                                  => 1,
    'sv_2bool'                                => 1,
    'sv_2bool_flags'                          => 1,
    'sv_2cv'                                  => 1,
    'sv_2io'                                  => 1,
    'sv_2iv_flags'                            => 1,
    'sv_2mortal'                              => 1,
    'sv_2nv_flags'                            => 1,
    'sv_2pv_flags'                            => 1,
    'sv_2pv_nolen'                            => 1,
    'sv_2pvbyte'                              => 1,
    'sv_2pvbyte_nolen'                        => 1,
    'sv_2pvutf8'                              => 1,
    'sv_2pvutf8_nolen'                        => 1,
    'sv_2uv_flags'                            => 1,
    'sv_backoff'                              => 1,
    'sv_bless'                                => 1,
    'sv_cat_decode'                           => 1,
    'sv_catpv'                                => 1,
    'sv_catpv_flags'                          => 1,
    'sv_catpv_mg'                             => 1,
    'sv_catpv_nomg'                           => 1,
    'sv_catpvf'                               => 1,
    'sv_catpvf_mg'                            => 1,
    'sv_catpvn'                               => 1,
    'sv_catpvn_flags'                         => 1,
    'sv_catpvn_mg'                            => 1,
    'sv_catpvn_nomg'                          => 1,
    'sv_catpvs'                               => 1,
    'sv_catpvs_flags'                         => 1,
    'sv_catpvs_mg'                            => 1,
    'sv_catpvs_nomg'                          => 1,
    'sv_catsv'                                => 1,
    'sv_catsv_flags'                          => 1,
    'sv_catsv_mg'                             => 1,
    'sv_catsv_nomg'                           => 1,
    'sv_chop'                                 => 1,
    'sv_clear'                                => 1,
    'sv_cmp'                                  => 1,
    'sv_cmp_flags'                            => 1,
    'sv_cmp_locale'                           => 1,
    'sv_cmp_locale_flags'                     => 1,
    'sv_collxfrm'                             => 1,
    'sv_collxfrm_flags'                       => 1,
    'sv_copypv'                               => 1,
    'sv_copypv_flags'                         => 1,
    'sv_copypv_nomg'                          => 1,
    'sv_dec'                                  => 1,
    'sv_dec_nomg'                             => 1,
    'sv_derived_from'                         => 1,
    'sv_derived_from_pv'                      => 1,
    'sv_derived_from_pvn'                     => 1,
    'sv_derived_from_sv'                      => 1,
    'sv_destroyable'                          => 1,
    'sv_does'                                 => 1,
    'sv_does_pv'                              => 1,
    'sv_does_pvn'                             => 1,
    'sv_does_sv'                              => 1,
    'sv_dump'                                 => 1,
    'sv_eq'                                   => 1,
    'sv_eq_flags'                             => 1,
    'sv_force_normal'                         => 1,
    'sv_force_normal_flags'                   => 1,
    'sv_free'                                 => 1,
    'sv_get_backrefs'                         => 1,
    'sv_gets'                                 => 1,
    'sv_grow'                                 => 1,
    'sv_inc'                                  => 1,
    'sv_inc_nomg'                             => 1,
    'sv_insert'                               => 1,
    'sv_insert_flags'                         => 1,
    'sv_isa'                                  => 1,
    'sv_isobject'                             => 1,
    'sv_iv'                                   => 1,
    'sv_len'                                  => 1,
    'sv_len_utf8'                             => 1,
    'sv_magic'                                => 1,
    'sv_magicext'                             => 1,
    'sv_mortalcopy'                           => 1,
    'sv_newmortal'                            => 1,
    'sv_newref'                               => 1,
    'sv_nolocking'                            => 1,
    'sv_nosharing'                            => 1,
    'sv_nounlocking'                          => 1,
    'sv_nv'                                   => 1,
    'sv_pos_b2u'                              => 1,
    'sv_pos_b2u_flags'                        => 1,
    'sv_pos_u2b'                              => 1,
    'sv_pos_u2b_flags'                        => 1,
    'sv_pv'                                   => 1,
    'sv_pvbyte'                               => 1,
    'sv_pvbyten'                              => 1,
    'sv_pvbyten_force'                        => 1,
    'sv_pvn'                                  => 1,
    'sv_pvn_force'                            => 1,
    'sv_pvn_force_flags'                      => 1,
    'sv_pvutf8'                               => 1,
    'sv_pvutf8n'                              => 1,
    'sv_pvutf8n_force'                        => 1,
    'sv_recode_to_utf8'                       => 1,
    'sv_ref'                                  => 1,
    'sv_reftype'                              => 1,
    'sv_replace'                              => 1,
    'sv_report_used'                          => 1,
    'sv_reset'                                => 1,
    'sv_rvweaken'                             => 1,
    'sv_set_undef'                            => 1,
    'sv_setiv'                                => 1,
    'sv_setiv_mg'                             => 1,
    'sv_setnv'                                => 1,
    'sv_setnv_mg'                             => 1,
    'sv_setpv'                                => 1,
    'sv_setpv_bufsize'                        => 1,
    'sv_setpv_mg'                             => 1,
    'sv_setpvf'                               => 1,
    'sv_setpvf_mg'                            => 1,
    'sv_setpviv'                              => 1,
    'sv_setpviv_mg'                           => 1,
    'sv_setpvn'                               => 1,
    'sv_setpvn_mg'                            => 1,
    'sv_setpvs'                               => 1,
    'sv_setpvs_mg'                            => 1,
    'sv_setref_iv'                            => 1,
    'sv_setref_nv'                            => 1,
    'sv_setref_pv'                            => 1,
    'sv_setref_pvn'                           => 1,
    'sv_setref_pvs'                           => 1,
    'sv_setref_uv'                            => 1,
    'sv_setsv'                                => 1,
    'sv_setsv_flags'                          => 1,
    'sv_setsv_mg'                             => 1,
    'sv_setsv_nomg'                           => 1,
    'sv_setuv'                                => 1,
    'sv_setuv_mg'                             => 1,
    'sv_taint'                                => 1,
    'sv_tainted'                              => 1,
    'sv_true'                                 => 1,
    'sv_uni_display'                          => 1,
    'sv_unmagic'                              => 1,
    'sv_unmagicext'                           => 1,
    'sv_unref'                                => 1,
    'sv_unref_flags'                          => 1,
    'sv_untaint'                              => 1,
    'sv_upgrade'                              => 1,
    'sv_usepvn'                               => 1,
    'sv_usepvn_flags'                         => 1,
    'sv_usepvn_mg'                            => 1,
    'sv_utf8_decode'                          => 1,
    'sv_utf8_downgrade'                       => 1,
    'sv_utf8_encode'                          => 1,
    'sv_utf8_upgrade'                         => 1,
    'sv_utf8_upgrade_flags'                   => 1,
    'sv_utf8_upgrade_flags_grow'              => 1,
    'sv_utf8_upgrade_nomg'                    => 1,
    'sv_uv'                                   => 1,
    'sv_vcatpvf'                              => 1,
    'sv_vcatpvf_mg'                           => 1,
    'sv_vcatpvfn'                             => 1,
    'sv_vcatpvfn_flags'                       => 1,
    'sv_vsetpvf'                              => 1,
    'sv_vsetpvf_mg'                           => 1,
    'sv_vsetpvfn'                             => 1,
    'svtype'                                  => 1,
    'sync_locale'                             => 1,
    'toFOLD'                                  => 1,
    'toFOLD_utf8'                             => 1,
    'toFOLD_utf8_safe'                        => 1,
    'toFOLD_uvchr'                            => 1,
    'toLOWER'                                 => 1,
    'toLOWER_L1'                              => 1,
    'toLOWER_LC'                              => 1,
    'toLOWER_utf8'                            => 1,
    'toLOWER_utf8_safe'                       => 1,
    'toLOWER_uvchr'                           => 1,
    'toTITLE'                                 => 1,
    'toTITLE_utf8'                            => 1,
    'toTITLE_utf8_safe'                       => 1,
    'toTITLE_uvchr'                           => 1,
    'toUPPER'                                 => 1,
    'toUPPER_utf8'                            => 1,
    'toUPPER_utf8_safe'                       => 1,
    'toUPPER_uvchr'                           => 1,
    'to_utf8_case'                            => 1,
    'to_utf8_fold'                            => 1,
    'to_utf8_lower'                           => 1,
    'to_utf8_title'                           => 1,
    'to_utf8_upper'                           => 1,
    'unpack_str'                              => 1,
    'unpackstring'                            => 1,
    'upg_version'                             => 1,
    'utf8_distance'                           => 1,
    'utf8_hop'                                => 1,
    'utf8_hop_back'                           => 1,
    'utf8_hop_forward'                        => 1,
    'utf8_hop_safe'                           => 1,
    'utf8_length'                             => 1,
    'utf8_to_bytes'                           => 1,
    'utf8_to_uvchr'                           => 1,
    'utf8_to_uvchr_buf'                       => 1,
    'utf8_to_uvuni'                           => 1,
    'utf8_to_uvuni_buf'                       => 1,
    'utf8n_to_uvchr'                          => 1,
    'utf8n_to_uvchr_error'                    => 1,
    'utf8n_to_uvuni'                          => 1,
    'uvchr_to_utf8'                           => 1,
    'uvchr_to_utf8_flags'                     => 1,
    'uvoffuni_to_utf8_flags'                  => 1,
    'uvuni_to_utf8_flags'                     => 1,
    'valid_utf8_to_uvchr'                     => 1,
    'vcmp'                                    => 1,
    'vcroak'                                  => 1,
    'vmess'                                   => 1,
    'vnormal'                                 => 1,
    'vnumify'                                 => 1,
    'vstringify'                              => 1,
    'vverify'                                 => 1,
    'vwarn'                                   => 1,
    'warn'                                    => 1,
    'warn_sv'                                 => 1,
    'wrap_op_checker'                         => 1,
};

$perlapinames_generated::FUNCTIONS_UNDOCUMENTED = undef;
$perlapinames_generated::FUNCTIONS_UNDOCUMENTED = {
    'GetVars'                  => 1,
    'Gv_AMupdate'              => 1,
    'PerlIO_clearerr'          => 1,
    'PerlIO_close'             => 1,
    'PerlIO_context_layers'    => 1,
    'PerlIO_eof'               => 1,
    'PerlIO_error'             => 1,
    'PerlIO_fileno'            => 1,
    'PerlIO_fill'              => 1,
    'PerlIO_flush'             => 1,
    'PerlIO_get_base'          => 1,
    'PerlIO_get_bufsiz'        => 1,
    'PerlIO_get_cnt'           => 1,
    'PerlIO_get_ptr'           => 1,
    'PerlIO_read'              => 1,
    'PerlIO_seek'              => 1,
    'PerlIO_set_cnt'           => 1,
    'PerlIO_set_ptrcnt'        => 1,
    'PerlIO_setlinebuf'        => 1,
    'PerlIO_stderr'            => 1,
    'PerlIO_stdin'             => 1,
    'PerlIO_stdout'            => 1,
    'PerlIO_tell'              => 1,
    'PerlIO_unread'            => 1,
    'PerlIO_write'             => 1,
    'amagic_call'              => 1,
    'amagic_deref_call'        => 1,
    'any_dup'                  => 1,
    'atfork_lock'              => 1,
    'atfork_unlock'            => 1,
    'av_arylen_p'              => 1,
    'av_iter_p'                => 1,
    'block_gimme'              => 1,
    'call_atexit'              => 1,
    'call_list'                => 1,
    'calloc'                   => 1,
    'cast_i32'                 => 1,
    'cast_iv'                  => 1,
    'cast_ulong'               => 1,
    'cast_uv'                  => 1,
    'ck_warner'                => 1,
    'ck_warner_d'              => 1,
    'ckwarn'                   => 1,
    'ckwarn_d'                 => 1,
    'clear_defarray'           => 1,
    'clone_params_del'         => 1,
    'clone_params_new'         => 1,
    'croak_memory_wrap'        => 1,
    'croak_nocontext'          => 1,
    'csighandler'              => 1,
    'cx_dump'                  => 1,
    'cx_dup'                   => 1,
    'cxinc'                    => 1,
    'deb'                      => 1,
    'deb_nocontext'            => 1,
    'debop'                    => 1,
    'debprofdump'              => 1,
    'debstack'                 => 1,
    'debstackptrs'             => 1,
    'delimcpy'                 => 1,
    'despatch_signals'         => 1,
    'die_nocontext'            => 1,
    'dirp_dup'                 => 1,
    'do_aspawn'                => 1,
    'do_binmode'               => 1,
    'do_close'                 => 1,
    'do_gv_dump'               => 1,
    'do_gvgv_dump'             => 1,
    'do_hv_dump'               => 1,
    'do_join'                  => 1,
    'do_magic_dump'            => 1,
    'do_op_dump'               => 1,
    'do_open'                  => 1,
    'do_open9'                 => 1,
    'do_openn'                 => 1,
    'do_pmop_dump'             => 1,
    'do_spawn'                 => 1,
    'do_spawn_nowait'          => 1,
    'do_sprintf'               => 1,
    'do_sv_dump'               => 1,
    'doing_taint'              => 1,
    'doref'                    => 1,
    'dounwind'                 => 1,
    'dowantarray'              => 1,
    'dump_eval'                => 1,
    'dump_form'                => 1,
    'dump_indent'              => 1,
    'dump_mstats'              => 1,
    'dump_sub'                 => 1,
    'dump_vindent'             => 1,
    'filter_add'               => 1,
    'filter_del'               => 1,
    'filter_read'              => 1,
    'foldEQ_latin1'            => 1,
    'form_nocontext'           => 1,
    'fp_dup'                   => 1,
    'fprintf_nocontext'        => 1,
    'free_global_struct'       => 1,
    'free_tmps'                => 1,
    'get_context'              => 1,
    'get_mstats'               => 1,
    'get_op_descs'             => 1,
    'get_op_names'             => 1,
    'get_ppaddr'               => 1,
    'get_vtbl'                 => 1,
    'gp_dup'                   => 1,
    'gp_free'                  => 1,
    'gp_ref'                   => 1,
    'gv_AVadd'                 => 1,
    'gv_HVadd'                 => 1,
    'gv_IOadd'                 => 1,
    'gv_SVadd'                 => 1,
    'gv_add_by_type'           => 1,
    'gv_autoload4'             => 1,
    'gv_autoload_pv'           => 1,
    'gv_autoload_pvn'          => 1,
    'gv_autoload_sv'           => 1,
    'gv_check'                 => 1,
    'gv_dump'                  => 1,
    'gv_efullname'             => 1,
    'gv_efullname3'            => 1,
    'gv_efullname4'            => 1,
    'gv_fetchfile'             => 1,
    'gv_fetchfile_flags'       => 1,
    'gv_fetchpv'               => 1,
    'gv_fetchpvn_flags'        => 1,
    'gv_fetchsv'               => 1,
    'gv_fullname'              => 1,
    'gv_fullname3'             => 1,
    'gv_fullname4'             => 1,
    'gv_handler'               => 1,
    'gv_name_set'              => 1,
    'he_dup'                   => 1,
    'hek_dup'                  => 1,
    'hv_common'                => 1,
    'hv_common_key_len'        => 1,
    'hv_delayfree_ent'         => 1,
    'hv_eiter_p'               => 1,
    'hv_eiter_set'             => 1,
    'hv_free_ent'              => 1,
    'hv_ksplit'                => 1,
    'hv_name_set'              => 1,
    'hv_placeholders_get'      => 1,
    'hv_placeholders_set'      => 1,
    'hv_rand_set'              => 1,
    'hv_riter_p'               => 1,
    'hv_riter_set'             => 1,
    'ibcmp_utf8'               => 1,
    'init_global_struct'       => 1,
    'init_stacks'              => 1,
    'init_tm'                  => 1,
    'instr'                    => 1,
    'is_lvalue_sub'            => 1,
    'leave_scope'              => 1,
    'load_module_nocontext'    => 1,
    'magic_dump'               => 1,
    'malloc'                   => 1,
    'markstack_grow'           => 1,
    'mess_nocontext'           => 1,
    'mfree'                    => 1,
    'mg_dup'                   => 1,
    'mg_size'                  => 1,
    'mini_mktime'              => 1,
    'moreswitches'             => 1,
    'mro_get_from_name'        => 1,
    'mro_get_private_data'     => 1,
    'mro_set_mro'              => 1,
    'mro_set_private_data'     => 1,
    'my_atof'                  => 1,
    'my_atof2'                 => 1,
    'my_bcopy'                 => 1,
    'my_bzero'                 => 1,
    'my_chsize'                => 1,
    'my_cxt_index'             => 1,
    'my_cxt_init'              => 1,
    'my_dirfd'                 => 1,
    'my_exit'                  => 1,
    'my_failure_exit'          => 1,
    'my_fflush_all'            => 1,
    'my_fork'                  => 1,
    'my_lstat'                 => 1,
    'my_memcmp'                => 1,
    'my_memset'                => 1,
    'my_pclose'                => 1,
    'my_popen'                 => 1,
    'my_popen_list'            => 1,
    'my_setenv'                => 1,
    'my_socketpair'            => 1,
    'my_stat'                  => 1,
    'my_strftime'              => 1,
    'newANONATTRSUB'           => 1,
    'newANONHASH'              => 1,
    'newANONLIST'              => 1,
    'newANONSUB'               => 1,
    'newATTRSUB'               => 1,
    'newAVREF'                 => 1,
    'newCVREF'                 => 1,
    'newFORM'                  => 1,
    'newGVREF'                 => 1,
    'newGVgen'                 => 1,
    'newGVgen_flags'           => 1,
    'newHVREF'                 => 1,
    'newHVhv'                  => 1,
    'newIO'                    => 1,
    'newMYSUB'                 => 1,
    'newPROG'                  => 1,
    'newRV'                    => 1,
    'newSUB'                   => 1,
    'newSVREF'                 => 1,
    'newSVpvf_nocontext'       => 1,
    'new_stackinfo'            => 1,
    'op_refcnt_lock'           => 1,
    'op_refcnt_unlock'         => 1,
    'parser_dup'               => 1,
    'perl_alloc_using'         => 1,
    'perl_clone_using'         => 1,
    'pmop_dump'                => 1,
    'pop_scope'                => 1,
    'pregcomp'                 => 1,
    'pregexec'                 => 1,
    'pregfree'                 => 1,
    'pregfree2'                => 1,
    'printf_nocontext'         => 1,
    'ptr_table_fetch'          => 1,
    'ptr_table_free'           => 1,
    'ptr_table_new'            => 1,
    'ptr_table_split'          => 1,
    'ptr_table_store'          => 1,
    'push_scope'               => 1,
    're_compile'               => 1,
    're_dup_guts'              => 1,
    're_intuit_start'          => 1,
    're_intuit_string'         => 1,
    'realloc'                  => 1,
    'reentrant_free'           => 1,
    'reentrant_init'           => 1,
    'reentrant_retry'          => 1,
    'reentrant_size'           => 1,
    'ref'                      => 1,
    'reg_named_buff_all'       => 1,
    'reg_named_buff_exists'    => 1,
    'reg_named_buff_fetch'     => 1,
    'reg_named_buff_firstkey'  => 1,
    'reg_named_buff_nextkey'   => 1,
    'reg_named_buff_scalar'    => 1,
    'regdump'                  => 1,
    'regdupe_internal'         => 1,
    'regexec_flags'            => 1,
    'regfree_internal'         => 1,
    'reginitcolors'            => 1,
    'regnext'                  => 1,
    'repeatcpy'                => 1,
    'rsignal'                  => 1,
    'rsignal_state'            => 1,
    'runops_debug'             => 1,
    'runops_standard'          => 1,
    'rvpv_dup'                 => 1,
    'safesyscalloc'            => 1,
    'safesysfree'              => 1,
    'safesysmalloc'            => 1,
    'safesysrealloc'           => 1,
    'save_I16'                 => 1,
    'save_I32'                 => 1,
    'save_I8'                  => 1,
    'save_adelete'             => 1,
    'save_aelem'               => 1,
    'save_aelem_flags'         => 1,
    'save_alloc'               => 1,
    'save_aptr'                => 1,
    'save_ary'                 => 1,
    'save_bool'                => 1,
    'save_clearsv'             => 1,
    'save_delete'              => 1,
    'save_destructor'          => 1,
    'save_destructor_x'        => 1,
    'save_freeop'              => 1,
    'save_freepv'              => 1,
    'save_freesv'              => 1,
    'save_generic_pvref'       => 1,
    'save_generic_svref'       => 1,
    'save_hash'                => 1,
    'save_hdelete'             => 1,
    'save_helem'               => 1,
    'save_helem_flags'         => 1,
    'save_hints'               => 1,
    'save_hptr'                => 1,
    'save_int'                 => 1,
    'save_item'                => 1,
    'save_iv'                  => 1,
    'save_list'                => 1,
    'save_long'                => 1,
    'save_mortalizesv'         => 1,
    'save_nogv'                => 1,
    'save_op'                  => 1,
    'save_padsv_and_mortalize' => 1,
    'save_pptr'                => 1,
    'save_pushi32ptr'          => 1,
    'save_pushptr'             => 1,
    'save_pushptrptr'          => 1,
    'save_re_context'          => 1,
    'save_scalar'              => 1,
    'save_set_svflags'         => 1,
    'save_shared_pvref'        => 1,
    'save_sptr'                => 1,
    'save_svref'               => 1,
    'save_vptr'                => 1,
    'savestack_grow'           => 1,
    'savestack_grow_cnt'       => 1,
    'scan_num'                 => 1,
    'scan_vstring'             => 1,
    'seed'                     => 1,
    'set_context'              => 1,
    'set_numeric_local'        => 1,
    'set_numeric_radix'        => 1,
    'set_numeric_standard'     => 1,
    'share_hek'                => 1,
    'si_dup'                   => 1,
    'ss_dup'                   => 1,
    'stack_grow'               => 1,
    'start_subparse'           => 1,
    'str_to_version'           => 1,
    'sv_2iv'                   => 1,
    'sv_2pv'                   => 1,
    'sv_2uv'                   => 1,
    'sv_catpvf_mg_nocontext'   => 1,
    'sv_catpvf_nocontext'      => 1,
    'sv_dup'                   => 1,
    'sv_dup_inc'               => 1,
    'sv_peek'                  => 1,
    'sv_pvn_nomg'              => 1,
    'sv_setpvf_mg_nocontext'   => 1,
    'sv_setpvf_nocontext'      => 1,
    'sys_init'                 => 1,
    'sys_init3'                => 1,
    'sys_intern_clear'         => 1,
    'sys_intern_dup'           => 1,
    'sys_intern_init'          => 1,
    'sys_term'                 => 1,
    'taint_env'                => 1,
    'taint_proper'             => 1,
    'unlnk'                    => 1,
    'unsharepvn'               => 1,
    'utf16_to_utf8'            => 1,
    'utf16_to_utf8_reversed'   => 1,
    'uvuni_to_utf8'            => 1,
    'vdeb'                     => 1,
    'vform'                    => 1,
    'vload_module'             => 1,
    'vnewSVpvf'                => 1,
    'vwarner'                  => 1,
    'warn_nocontext'           => 1,
    'warner'                   => 1,
    'warner_nocontext'         => 1,
    'whichsig'                 => 1,
    'whichsig_pv'              => 1,
    'whichsig_pvn'             => 1,
    'whichsig_sv'              => 1,
};

$perlapinames_generated::VARIABLES_DOCUMENTED = undef;
$perlapinames_generated::VARIABLES_DOCUMENTED = {
    'av'               => 1,
    'c_backtrace_dump' => 1,
    'cv'               => 1,
    'cvn_flags'        => 1,
    'hv'               => 1,
    'sv'               => 1,
};

$perlapinames_generated::VARIABLES_UNDOCUMENTED = undef;
$perlapinames_generated::VARIABLES_UNDOCUMENTED = {
    'context'          => 1,
    'mstats'           => 1,
    'numeric_local'    => 1,
    'numeric_radix'    => 1,
    'numeric_standard' => 1,
    'op_descs'         => 1,
    'op_names'         => 1,
    'ppaddr'           => 1,
    'vtbl'             => 1,
};

1;    # end of package
