#ifndef __CPP__INCLUDED__perlgsl_h
#define __CPP__INCLUDED__perlgsl_h 0.001_000

#include <Perl/Structure/GSLMatrix.cpp>  // -> GSLMatrix.h
//#include <RPerl/Operation/Expression/Operator/GSLFunctions.cpp>  // -> GSLFunctions.h  // NEED DELETE, RPERL REFACTOR

#endif
