mv t/lib/Perl/Types/Test/SSE/program_00_good.pl t/lib_disabled/Perl/Types/Test/SSE/program_00_good.pl
mv t/lib/Perl/Types/Test/SSE/program_01_good.pl t/lib_disabled/Perl/Types/Test/SSE/program_01_good.pl
mv t/lib/Perl/Types/Test/ScopeTypeNameValue/program_01_good.pl t/lib_disabled/Perl/Types/Test/ScopeTypeNameValue/program_01_good.pl
mv t/lib/Perl/Types/Test/ScopeTypeNameValue/program_02_good.pl t/lib_disabled/Perl/Types/Test/ScopeTypeNameValue/program_02_good.pl
mv t/lib/Perl/Types/Test/ScopeTypeNameValue/program_03_good.pl t/lib_disabled/Perl/Types/Test/ScopeTypeNameValue/program_03_good.pl
mv t/lib/Perl/Types/Test/ScopeTypeNameValue/program_04_good.pl t/lib_disabled/Perl/Types/Test/ScopeTypeNameValue/program_04_good.pl
mv t/lib/Perl/Types/Test/ScopeTypeNameValue/program_06_good.pl t/lib_disabled/Perl/Types/Test/ScopeTypeNameValue/program_06_good.pl
mv t/lib/Perl/Types/Test/ScopeTypeNameValue/program_07_good.pl t/lib_disabled/Perl/Types/Test/ScopeTypeNameValue/program_07_good.pl
mv t/lib/Perl/Types/Test/ScopeTypeNameValue/program_08_good.pl t/lib_disabled/Perl/Types/Test/ScopeTypeNameValue/program_08_good.pl
mv t/lib/Perl/Types/Test/ScopeTypeNameValue/program_09_good.pl t/lib_disabled/Perl/Types/Test/ScopeTypeNameValue/program_09_good.pl
mv t/lib/Perl/Types/Test/ScopeTypeNameValue/program_10_good.pl t/lib_disabled/Perl/Types/Test/ScopeTypeNameValue/program_10_good.pl
mv t/lib/Perl/Types/Test/ScopeTypeNameValue/program_11_good.pl t/lib_disabled/Perl/Types/Test/ScopeTypeNameValue/program_11_good.pl
mv t/lib/Perl/Types/Test/ScopeTypeNameValue/program_12_good.pl t/lib_disabled/Perl/Types/Test/ScopeTypeNameValue/program_12_good.pl
mv t/lib/Perl/Types/Test/ScopeTypeNameValue/program_13_good.pl t/lib_disabled/Perl/Types/Test/ScopeTypeNameValue/program_13_good.pl
mv t/lib/Perl/Types/Test/ScopeTypeNameValue/program_14_good.pl t/lib_disabled/Perl/Types/Test/ScopeTypeNameValue/program_14_good.pl
mv t/lib/Perl/Types/Test/ScopeTypeNameValue/program_30_good.pl t/lib_disabled/Perl/Types/Test/ScopeTypeNameValue/program_30_good.pl
mv t/lib/Perl/Types/Test/ScopeTypeNameValue/program_31_good.pl t/lib_disabled/Perl/Types/Test/ScopeTypeNameValue/program_31_good.pl
mv t/lib/Perl/Types/Test/ScopeTypeNameValue/program_32_good.pl t/lib_disabled/Perl/Types/Test/ScopeTypeNameValue/program_32_good.pl
mv t/lib/Perl/Types/Test/ScopeTypeNameValue/program_33_good.pl t/lib_disabled/Perl/Types/Test/ScopeTypeNameValue/program_33_good.pl
mv t/lib/Perl/Types/Test/ScopeTypeNameValue/program_34_good.pl t/lib_disabled/Perl/Types/Test/ScopeTypeNameValue/program_34_good.pl
mv t/lib/Perl/Types/Test/ScopeTypeNameValue/program_40_good.pl t/lib_disabled/Perl/Types/Test/ScopeTypeNameValue/program_40_good.pl
mv t/lib/Perl/Types/Test/ScopeTypeNameValue/program_60_good.pl t/lib_disabled/Perl/Types/Test/ScopeTypeNameValue/program_60_good.pl
mv t/lib/Perl/Types/Test/Subclass/mysubclasser_a_good.pl t/lib_disabled/Perl/Types/Test/Subclass/mysubclasser_a_good.pl
mv t/lib/Perl/Types/Test/Subclass/mysubclasser_b_good.pl t/lib_disabled/Perl/Types/Test/Subclass/mysubclasser_b_good.pl
mv t/lib/Perl/Types/Test/Subclass/mysubclassers_cde_good.pl t/lib_disabled/Perl/Types/Test/Subclass/mysubclassers_cde_good.pl
mv t/lib/Perl/Types/Test/Subclass/mysubclassers_fg_bad_00.pl t/lib_disabled/Perl/Types/Test/Subclass/mysubclassers_fg_bad_00.pl
mv t/lib/Perl/Types/Test/Subclass/mysubclassers_fg_bad_01.pl t/lib_disabled/Perl/Types/Test/Subclass/mysubclassers_fg_bad_01.pl
mv t/lib/Perl/Types/Test/Subclass/mysubclassers_fg_bad_02.pl t/lib_disabled/Perl/Types/Test/Subclass/mysubclassers_fg_bad_02.pl
mv t/lib/Perl/Types/Test/Subclass/mysubclassers_fg_good.pl t/lib_disabled/Perl/Types/Test/Subclass/mysubclassers_fg_good.pl
mv t/lib/Perl/Types/Test/TypeCheckingOn/arrayref_bad_00.pl t/lib_disabled/Perl/Types/Test/TypeCheckingOn/arrayref_bad_00.pl
mv t/lib/Perl/Types/Test/TypeCheckingOn/arrayref_bad_01.pl t/lib_disabled/Perl/Types/Test/TypeCheckingOn/arrayref_bad_01.pl
mv t/lib/Perl/Types/Test/TypeCheckingOn/arrayref_bad_02.pl t/lib_disabled/Perl/Types/Test/TypeCheckingOn/arrayref_bad_02.pl
mv t/lib/Perl/Types/Test/TypeCheckingOn/arrayref_bad_03.pl t/lib_disabled/Perl/Types/Test/TypeCheckingOn/arrayref_bad_03.pl
mv t/lib/Perl/Types/Test/TypeCheckingOn/hashref_bad_00.pl t/lib_disabled/Perl/Types/Test/TypeCheckingOn/hashref_bad_00.pl
mv t/lib/Perl/Types/Test/TypeCheckingOn/hashref_bad_01.pl t/lib_disabled/Perl/Types/Test/TypeCheckingOn/hashref_bad_01.pl
mv t/lib/Perl/Types/Test/TypeCheckingOn/hashref_bad_02.pl t/lib_disabled/Perl/Types/Test/TypeCheckingOn/hashref_bad_02.pl
mv t/lib/Perl/Types/Test/TypeCheckingOn/hashref_bad_03.pl t/lib_disabled/Perl/Types/Test/TypeCheckingOn/hashref_bad_03.pl
mv t/lib/Perl/Types/Test/TypeCheckingOn/integer_arrayref_bad_00.pl t/lib_disabled/Perl/Types/Test/TypeCheckingOn/integer_arrayref_bad_00.pl
mv t/lib/Perl/Types/Test/TypeCheckingOn/integer_arrayref_bad_01.pl t/lib_disabled/Perl/Types/Test/TypeCheckingOn/integer_arrayref_bad_01.pl
mv t/lib/Perl/Types/Test/TypeCheckingOn/integer_arrayref_bad_02.pl t/lib_disabled/Perl/Types/Test/TypeCheckingOn/integer_arrayref_bad_02.pl
mv t/lib/Perl/Types/Test/TypeCheckingOn/integer_arrayref_bad_03.pl t/lib_disabled/Perl/Types/Test/TypeCheckingOn/integer_arrayref_bad_03.pl
mv t/lib/Perl/Types/Test/TypeCheckingOn/integer_bad_00.pl t/lib_disabled/Perl/Types/Test/TypeCheckingOn/integer_bad_00.pl
mv t/lib/Perl/Types/Test/TypeCheckingOn/integer_bad_01.pl t/lib_disabled/Perl/Types/Test/TypeCheckingOn/integer_bad_01.pl
mv t/lib/Perl/Types/Test/TypeCheckingOn/integer_bad_02.pl t/lib_disabled/Perl/Types/Test/TypeCheckingOn/integer_bad_02.pl
mv t/lib/Perl/Types/Test/TypeCheckingOn/integer_bad_03.pl t/lib_disabled/Perl/Types/Test/TypeCheckingOn/integer_bad_03.pl
mv t/lib/Perl/Types/Test/TypeCheckingOn/integer_hashref_bad_00.pl t/lib_disabled/Perl/Types/Test/TypeCheckingOn/integer_hashref_bad_00.pl
mv t/lib/Perl/Types/Test/TypeCheckingOn/integer_hashref_bad_01.pl t/lib_disabled/Perl/Types/Test/TypeCheckingOn/integer_hashref_bad_01.pl
mv t/lib/Perl/Types/Test/TypeCheckingOn/integer_hashref_bad_02.pl t/lib_disabled/Perl/Types/Test/TypeCheckingOn/integer_hashref_bad_02.pl
mv t/lib/Perl/Types/Test/TypeCheckingOn/integer_hashref_bad_03.pl t/lib_disabled/Perl/Types/Test/TypeCheckingOn/integer_hashref_bad_03.pl
mv t/lib/Perl/Types/Test/TypeCheckingOn/number_arrayref_bad_00.pl t/lib_disabled/Perl/Types/Test/TypeCheckingOn/number_arrayref_bad_00.pl
mv t/lib/Perl/Types/Test/TypeCheckingOn/number_arrayref_bad_01.pl t/lib_disabled/Perl/Types/Test/TypeCheckingOn/number_arrayref_bad_01.pl
mv t/lib/Perl/Types/Test/TypeCheckingOn/number_arrayref_bad_02.pl t/lib_disabled/Perl/Types/Test/TypeCheckingOn/number_arrayref_bad_02.pl
mv t/lib/Perl/Types/Test/TypeCheckingOn/number_arrayref_bad_03.pl t/lib_disabled/Perl/Types/Test/TypeCheckingOn/number_arrayref_bad_03.pl
mv t/lib/Perl/Types/Test/TypeCheckingOn/number_arrayrefs_bad_00.pl t/lib_disabled/Perl/Types/Test/TypeCheckingOn/number_arrayrefs_bad_00.pl
mv t/lib/Perl/Types/Test/TypeCheckingOn/number_arrayrefs_bad_01.pl t/lib_disabled/Perl/Types/Test/TypeCheckingOn/number_arrayrefs_bad_01.pl
mv t/lib/Perl/Types/Test/TypeCheckingOn/number_bad_00.pl t/lib_disabled/Perl/Types/Test/TypeCheckingOn/number_bad_00.pl
mv t/lib/Perl/Types/Test/TypeCheckingOn/number_bad_01.pl t/lib_disabled/Perl/Types/Test/TypeCheckingOn/number_bad_01.pl
mv t/lib/Perl/Types/Test/TypeCheckingOn/number_bad_02.pl t/lib_disabled/Perl/Types/Test/TypeCheckingOn/number_bad_02.pl
mv t/lib/Perl/Types/Test/TypeCheckingOn/number_bad_03.pl t/lib_disabled/Perl/Types/Test/TypeCheckingOn/number_bad_03.pl
mv t/lib/Perl/Types/Test/TypeCheckingOn/number_hashref_bad_00.pl t/lib_disabled/Perl/Types/Test/TypeCheckingOn/number_hashref_bad_00.pl
mv t/lib/Perl/Types/Test/TypeCheckingOn/number_hashref_bad_01.pl t/lib_disabled/Perl/Types/Test/TypeCheckingOn/number_hashref_bad_01.pl
mv t/lib/Perl/Types/Test/TypeCheckingOn/number_hashref_bad_02.pl t/lib_disabled/Perl/Types/Test/TypeCheckingOn/number_hashref_bad_02.pl
mv t/lib/Perl/Types/Test/TypeCheckingOn/number_hashref_bad_03.pl t/lib_disabled/Perl/Types/Test/TypeCheckingOn/number_hashref_bad_03.pl
mv t/lib/Perl/Types/Test/TypeCheckingOn/number_hashrefs_bad_00.pl t/lib_disabled/Perl/Types/Test/TypeCheckingOn/number_hashrefs_bad_00.pl
mv t/lib/Perl/Types/Test/TypeCheckingOn/number_hashrefs_bad_01.pl t/lib_disabled/Perl/Types/Test/TypeCheckingOn/number_hashrefs_bad_01.pl
mv t/lib/Perl/Types/Test/TypeCheckingOn/string_arrayref_bad_00.pl t/lib_disabled/Perl/Types/Test/TypeCheckingOn/string_arrayref_bad_00.pl
mv t/lib/Perl/Types/Test/TypeCheckingOn/string_arrayref_bad_01.pl t/lib_disabled/Perl/Types/Test/TypeCheckingOn/string_arrayref_bad_01.pl
mv t/lib/Perl/Types/Test/TypeCheckingOn/string_arrayref_bad_02.pl t/lib_disabled/Perl/Types/Test/TypeCheckingOn/string_arrayref_bad_02.pl
mv t/lib/Perl/Types/Test/TypeCheckingOn/string_arrayref_bad_03.pl t/lib_disabled/Perl/Types/Test/TypeCheckingOn/string_arrayref_bad_03.pl
mv t/lib/Perl/Types/Test/TypeCheckingOn/string_bad_00.pl t/lib_disabled/Perl/Types/Test/TypeCheckingOn/string_bad_00.pl
mv t/lib/Perl/Types/Test/TypeCheckingOn/string_bad_01.pl t/lib_disabled/Perl/Types/Test/TypeCheckingOn/string_bad_01.pl
mv t/lib/Perl/Types/Test/TypeCheckingOn/string_bad_02.pl t/lib_disabled/Perl/Types/Test/TypeCheckingOn/string_bad_02.pl
mv t/lib/Perl/Types/Test/TypeCheckingOn/string_bad_03.pl t/lib_disabled/Perl/Types/Test/TypeCheckingOn/string_bad_03.pl
mv t/lib/Perl/Types/Test/TypeCheckingOn/string_hashref_bad_00.pl t/lib_disabled/Perl/Types/Test/TypeCheckingOn/string_hashref_bad_00.pl
mv t/lib/Perl/Types/Test/TypeCheckingOn/string_hashref_bad_01.pl t/lib_disabled/Perl/Types/Test/TypeCheckingOn/string_hashref_bad_01.pl
mv t/lib/Perl/Types/Test/TypeCheckingOn/string_hashref_bad_02.pl t/lib_disabled/Perl/Types/Test/TypeCheckingOn/string_hashref_bad_02.pl
mv t/lib/Perl/Types/Test/TypeCheckingOn/string_hashref_bad_03.pl t/lib_disabled/Perl/Types/Test/TypeCheckingOn/string_hashref_bad_03.pl
mv t/lib/Perl/Types/Test/TypeCheckingTrace/arrayref_bad_00.pl t/lib_disabled/Perl/Types/Test/TypeCheckingTrace/arrayref_bad_00.pl
mv t/lib/Perl/Types/Test/TypeCheckingTrace/arrayref_bad_01.pl t/lib_disabled/Perl/Types/Test/TypeCheckingTrace/arrayref_bad_01.pl
mv t/lib/Perl/Types/Test/TypeCheckingTrace/arrayref_bad_02.pl t/lib_disabled/Perl/Types/Test/TypeCheckingTrace/arrayref_bad_02.pl
mv t/lib/Perl/Types/Test/TypeCheckingTrace/arrayref_bad_03.pl t/lib_disabled/Perl/Types/Test/TypeCheckingTrace/arrayref_bad_03.pl
mv t/lib/Perl/Types/Test/TypeCheckingTrace/hashref_bad_00.pl t/lib_disabled/Perl/Types/Test/TypeCheckingTrace/hashref_bad_00.pl
mv t/lib/Perl/Types/Test/TypeCheckingTrace/hashref_bad_01.pl t/lib_disabled/Perl/Types/Test/TypeCheckingTrace/hashref_bad_01.pl
mv t/lib/Perl/Types/Test/TypeCheckingTrace/hashref_bad_02.pl t/lib_disabled/Perl/Types/Test/TypeCheckingTrace/hashref_bad_02.pl
mv t/lib/Perl/Types/Test/TypeCheckingTrace/hashref_bad_03.pl t/lib_disabled/Perl/Types/Test/TypeCheckingTrace/hashref_bad_03.pl
mv t/lib/Perl/Types/Test/TypeCheckingTrace/integer_arrayref_bad_00.pl t/lib_disabled/Perl/Types/Test/TypeCheckingTrace/integer_arrayref_bad_00.pl
mv t/lib/Perl/Types/Test/TypeCheckingTrace/integer_arrayref_bad_01.pl t/lib_disabled/Perl/Types/Test/TypeCheckingTrace/integer_arrayref_bad_01.pl
mv t/lib/Perl/Types/Test/TypeCheckingTrace/integer_arrayref_bad_02.pl t/lib_disabled/Perl/Types/Test/TypeCheckingTrace/integer_arrayref_bad_02.pl
mv t/lib/Perl/Types/Test/TypeCheckingTrace/integer_arrayref_bad_03.pl t/lib_disabled/Perl/Types/Test/TypeCheckingTrace/integer_arrayref_bad_03.pl
mv t/lib/Perl/Types/Test/TypeCheckingTrace/integer_bad_00.pl t/lib_disabled/Perl/Types/Test/TypeCheckingTrace/integer_bad_00.pl
mv t/lib/Perl/Types/Test/TypeCheckingTrace/integer_bad_01.pl t/lib_disabled/Perl/Types/Test/TypeCheckingTrace/integer_bad_01.pl
mv t/lib/Perl/Types/Test/TypeCheckingTrace/integer_bad_02.pl t/lib_disabled/Perl/Types/Test/TypeCheckingTrace/integer_bad_02.pl
mv t/lib/Perl/Types/Test/TypeCheckingTrace/integer_bad_03.pl t/lib_disabled/Perl/Types/Test/TypeCheckingTrace/integer_bad_03.pl
mv t/lib/Perl/Types/Test/TypeCheckingTrace/integer_hashref_bad_00.pl t/lib_disabled/Perl/Types/Test/TypeCheckingTrace/integer_hashref_bad_00.pl
mv t/lib/Perl/Types/Test/TypeCheckingTrace/integer_hashref_bad_01.pl t/lib_disabled/Perl/Types/Test/TypeCheckingTrace/integer_hashref_bad_01.pl
mv t/lib/Perl/Types/Test/TypeCheckingTrace/integer_hashref_bad_02.pl t/lib_disabled/Perl/Types/Test/TypeCheckingTrace/integer_hashref_bad_02.pl
mv t/lib/Perl/Types/Test/TypeCheckingTrace/integer_hashref_bad_03.pl t/lib_disabled/Perl/Types/Test/TypeCheckingTrace/integer_hashref_bad_03.pl
mv t/lib/Perl/Types/Test/TypeCheckingTrace/number_arrayref_bad_00.pl t/lib_disabled/Perl/Types/Test/TypeCheckingTrace/number_arrayref_bad_00.pl
mv t/lib/Perl/Types/Test/TypeCheckingTrace/number_arrayref_bad_01.pl t/lib_disabled/Perl/Types/Test/TypeCheckingTrace/number_arrayref_bad_01.pl
mv t/lib/Perl/Types/Test/TypeCheckingTrace/number_arrayref_bad_02.pl t/lib_disabled/Perl/Types/Test/TypeCheckingTrace/number_arrayref_bad_02.pl
mv t/lib/Perl/Types/Test/TypeCheckingTrace/number_arrayref_bad_03.pl t/lib_disabled/Perl/Types/Test/TypeCheckingTrace/number_arrayref_bad_03.pl
mv t/lib/Perl/Types/Test/TypeCheckingTrace/number_arrayrefs_bad_00.pl t/lib_disabled/Perl/Types/Test/TypeCheckingTrace/number_arrayrefs_bad_00.pl
mv t/lib/Perl/Types/Test/TypeCheckingTrace/number_arrayrefs_bad_01.pl t/lib_disabled/Perl/Types/Test/TypeCheckingTrace/number_arrayrefs_bad_01.pl
mv t/lib/Perl/Types/Test/TypeCheckingTrace/number_bad_00.pl t/lib_disabled/Perl/Types/Test/TypeCheckingTrace/number_bad_00.pl
mv t/lib/Perl/Types/Test/TypeCheckingTrace/number_bad_01.pl t/lib_disabled/Perl/Types/Test/TypeCheckingTrace/number_bad_01.pl
mv t/lib/Perl/Types/Test/TypeCheckingTrace/number_bad_02.pl t/lib_disabled/Perl/Types/Test/TypeCheckingTrace/number_bad_02.pl
mv t/lib/Perl/Types/Test/TypeCheckingTrace/number_bad_03.pl t/lib_disabled/Perl/Types/Test/TypeCheckingTrace/number_bad_03.pl
mv t/lib/Perl/Types/Test/TypeCheckingTrace/number_hashref_bad_00.pl t/lib_disabled/Perl/Types/Test/TypeCheckingTrace/number_hashref_bad_00.pl
mv t/lib/Perl/Types/Test/TypeCheckingTrace/number_hashref_bad_01.pl t/lib_disabled/Perl/Types/Test/TypeCheckingTrace/number_hashref_bad_01.pl
mv t/lib/Perl/Types/Test/TypeCheckingTrace/number_hashref_bad_02.pl t/lib_disabled/Perl/Types/Test/TypeCheckingTrace/number_hashref_bad_02.pl
mv t/lib/Perl/Types/Test/TypeCheckingTrace/number_hashref_bad_03.pl t/lib_disabled/Perl/Types/Test/TypeCheckingTrace/number_hashref_bad_03.pl
mv t/lib/Perl/Types/Test/TypeCheckingTrace/number_hashrefs_bad_00.pl t/lib_disabled/Perl/Types/Test/TypeCheckingTrace/number_hashrefs_bad_00.pl
mv t/lib/Perl/Types/Test/TypeCheckingTrace/number_hashrefs_bad_01.pl t/lib_disabled/Perl/Types/Test/TypeCheckingTrace/number_hashrefs_bad_01.pl
mv t/lib/Perl/Types/Test/TypeCheckingTrace/string_arrayref_bad_00.pl t/lib_disabled/Perl/Types/Test/TypeCheckingTrace/string_arrayref_bad_00.pl
mv t/lib/Perl/Types/Test/TypeCheckingTrace/string_arrayref_bad_01.pl t/lib_disabled/Perl/Types/Test/TypeCheckingTrace/string_arrayref_bad_01.pl
mv t/lib/Perl/Types/Test/TypeCheckingTrace/string_arrayref_bad_02.pl t/lib_disabled/Perl/Types/Test/TypeCheckingTrace/string_arrayref_bad_02.pl
mv t/lib/Perl/Types/Test/TypeCheckingTrace/string_arrayref_bad_03.pl t/lib_disabled/Perl/Types/Test/TypeCheckingTrace/string_arrayref_bad_03.pl
mv t/lib/Perl/Types/Test/TypeCheckingTrace/string_bad_00.pl t/lib_disabled/Perl/Types/Test/TypeCheckingTrace/string_bad_00.pl
mv t/lib/Perl/Types/Test/TypeCheckingTrace/string_bad_01.pl t/lib_disabled/Perl/Types/Test/TypeCheckingTrace/string_bad_01.pl
mv t/lib/Perl/Types/Test/TypeCheckingTrace/string_bad_02.pl t/lib_disabled/Perl/Types/Test/TypeCheckingTrace/string_bad_02.pl
mv t/lib/Perl/Types/Test/TypeCheckingTrace/string_bad_03.pl t/lib_disabled/Perl/Types/Test/TypeCheckingTrace/string_bad_03.pl
mv t/lib/Perl/Types/Test/TypeCheckingTrace/string_hashref_bad_00.pl t/lib_disabled/Perl/Types/Test/TypeCheckingTrace/string_hashref_bad_00.pl
mv t/lib/Perl/Types/Test/TypeCheckingTrace/string_hashref_bad_01.pl t/lib_disabled/Perl/Types/Test/TypeCheckingTrace/string_hashref_bad_01.pl
mv t/lib/Perl/Types/Test/TypeCheckingTrace/string_hashref_bad_02.pl t/lib_disabled/Perl/Types/Test/TypeCheckingTrace/string_hashref_bad_02.pl
mv t/lib/Perl/Types/Test/TypeCheckingTrace/string_hashref_bad_03.pl t/lib_disabled/Perl/Types/Test/TypeCheckingTrace/string_hashref_bad_03.pl
