#ifndef __CPP__INCLUDED__perlgmp_h
#define __CPP__INCLUDED__perlgmp_h 0.004_000

#include <Perl/Type/GMPInteger.cpp>  // -> GMPInteger.h
// #include <RPerl/Operation/Expression/Operator/GMPFunctions.cpp>  // -> GMPFunctions.h  // NEED DELETE, RPERL REFACTOR

#endif
