cd lib
rm ./MyClass_Good.* _MyClass_Bad.pm 
rm rperloperations.h
rm rperloperations.pm
rm rperloptions.pm 
rm rperlrules.pm 
rm rperlstandalone.h 
cd RPerl
rm AfterSubclass.pm 
rm -Rf Algorithm*
rm CodeBlock.pm 
rm Compiler.pm 
rm CompileUnit.pm 
rm Exporter.pm 
rm Grammar.eyp 
rm GrammarMedium.eyp 
rm Grammar.output 
rm Grammar.pm 
rm GrammarRule.pm 
rm InputOutput.pm 
rm -rf InputOutput/
rm -Rf Learning*
rm NonGenerator.pm 
rm -Rf Operation*
rm Parser.pm 
rm -Rf Support*
cd Test/
rm -Rf Operator*
rm -Rf Conditional/ Critic/ DatabaseMongoDB/ Exporter/ Expression/ Header/ Include/ Loop* Method* Module/ Pod/ Program/ Shebang/ Statement/ SubExpression* Subroutine VersionNumber/
rm -Rf PerlAPINames/
rm -Rf Reserved/
cd ..
rm -Rf CodeBlock/
cd CompileUnit/
rm Constant.pm 
rm Critic.pm 
rm Include.pm 
rm Module.pm
rm Module.pmc.CPPOPS_DUALTYPES_TEMPLATE*
rm Program.pm 
cd Module/
rm Package.pm 
rm Header.pm 
rm -Rf Package/
rm -Rf Class
cd ..
cd ..
cd ..
cd ..
cd backup/
rm ./*
cd ..
cd docs
mv todo.txt ../
rm *
mv ../todo.txt ./
cd ..
cd script/
rm ./*.pl
rm -Rf 3rd_party_debugging/
rm -Rf demo
rm rperl rperl_installer.sh 
cd development/
mv GMP/ GSL/ namespaces_regenerate.pl OO/ perlapinames_regenerate.pl rperl_locallib_uninstall.sh ..
cd unused/
mv data_structure_* data_type_* ../../
cd ..
rm -Rf ./*
cd ..
mv * development/
cd development/
rm -Rf OO
mkdir unused
mv data_* unused/
cd ..
cd ..
cd t
rm 1* 2*
cd ..
rm Artistic 
rm CONTRIBUTING 
rm ASSIGNMENT.pdf 
rm MANIFEST.SKIP 
rm appveyor.yml 
rm .travis.yml 
rm AUTHORS 
rm Changes 
rm Copying 
rm EMPLOYERS.pdf 
rm INSTALL 
rm LICENSE 
rm Makefile.PL 
rm META.json 
rm README
rm README.md 
rm THANKS 
rm .gitmodules 
