# [[[ HEADER ]]]
package Perl::Structure::GraphReference;
use strict;
use warnings;
use Perl::Types;
our $VERSION = 0.001_000;

# [[[ OO INHERITANCE ]]]
use parent qw(Perl::Structure);
use Perl::Structure;

# [[[ OO PROPERTIES ]]]
our hashref $properties = {};

1;    # end of class
