using std::cout;  using std::cerr;  using std::endl;

#ifndef __CPP__INCLUDED__Perl__Structure__Hash_cpp
#define __CPP__INCLUDED__Perl__Structure__Hash_cpp 0.009_000

#include <Perl/Structure/Hash.h>		// -> NULL (relies on <unordered_map> being included via Inline::CPP's AUTO_INCLUDE config option)

#endif
